#!/bin/bash
#    destructs a zone (generally called by devzone_free/devzone_adm)
#    Copyright (C) 2008  Tim Spriggs - tim@tajinc.org
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

export PATH=/usr/gnu/bin:/bin:/sbin:/usr/bin:/usr/sbin

if [ "x$1" == "x-zone" ]; then
	export zone_name=`awk '/^'$2'[ ]*:/ {sub(":"," ");print $1}' /etc/devzone/user_zones.txt`
        if [ $? != 0 ] || [ x$zone_name == x ]; then
          exit -1
        fi
else
	if [ "x$2" == "x" ]; then
	  echo go away +
	  echo "$@"
	  exit -1
	fi
	if [ "x$3" != "x" ]; then
	  echo go away -
	  exit -1
	fi

	export g_user="$1"
	export g_uid="$2"

	## Check to see if a line like what we want exists
	grep :$g_user /etc/devzone/user_zones.txt | grep :$g_uid &> /dev/null
	if [ $? != 0 ]; then
	  exit -1
	fi

	## Get the name of the zone that goes with our user
	export zone_name=`awk '/:'$g_user'[ ]*':$g_uid'[ ]*$/ {sub(":"," ");print $1}' /etc/devzone/user_zones.txt`
	if [ "x$zone_name" == "x" ]; then
	  exit -1
	fi
fi

echo taking down zone: $zone_name

zonecfg -z $zone_name export 2>&1 | grep -v 'No such zone configured' > /dev/null
if [ $? == 0 ]; then
  export ZONE_STATE=`zoneadm -z $zone_name list -p | cut -d: -f3`
  case "$ZONE_STATE" in
    running)
      echo halting...
      zoneadm -z $zone_name halt
      echo uninstalling...
      zoneadm -z $zone_name uninstall -F
      echo deleting configuration...
      zonecfg -z $zone_name delete -F
      ;;
    installed|incomplete)
      echo uninstalling...
      zoneadm -z $zone_name uninstall -F
      echo deleting configuration...
      zonecfg -z $zone_name delete -F
      ;;
    configured)
      echo deleting configuration...
      zonecfg -z $zone_name delete -F
      ;;
    *)
      echo error: $zone_name is in $ZONE_STATE
      echo it should be in a running/installed/configured state
      exit -1
      ;;
  esac
fi

