#include "common.h"
#include "gpl.h"

int bootZone (const char *);
void enterZone (const char *, int, char **);

int
main (int argc, char **argv)
{
  char *reservedZoneName;
  char zoneState[30];

  init ();
  reservedZoneName = getReservedZoneName ();
  if (reservedZoneName == NULL)
    {
      destroy ();
      fprintf (stderr,
	       "Error: you must run " WANT_ZONE_CMD " to reserve a zone.\n");
      return -1;
    }

  getZoneState (reservedZoneName, (char *) zoneState);
  printf ("Zone state: %s\n", zoneState);

  if (strcmp (zoneState, "configured") == 0)
    {
      fprintf (stderr,
	       "Error: zone is not installed. Please re-initialize zone.\n");
      fprintf (stderr,
	       "Hint: you might try " FREE_ZONE_CMD ";" WANT_ZONE_CMD "\n");
    }

  if (strcmp (zoneState, "installed") == 0)
    {
      printf ("Zone is not running, bringing up zone...\n");
      bootZone (reservedZoneName);

      // Get new zone status...
      getZoneState (reservedZoneName, (char *) zoneState);
      printf ("Zone state: %s\n", zoneState);
    }

  // In theory, this is as far as this process will ever get.
  if (strcmp (zoneState, "running") == 0)
    {
      destroy ();
      printf ("Entering zone...\n");
      enterZone (reservedZoneName, argc, argv);	// will never return
    }

  if (strcmp (zoneState, "installed") == 0)
    {
      destroy ();
      fprintf (stderr, "Error: could not boot zone.\n");
      return -1;
    }

  fprintf (stderr, "Error: zone is in an unknown state: %s\n", zoneState);
  destroy ();

  return -1;
}

int
bootZone (const char *zoneName)
{
  int childPID;
  int exitStatus;

  childPID = fork ();
  if (!childPID)
    {				// child
      setreuid (0, 0);

      // boot the zone
      execl ("/usr/sbin/zoneadm", "zoneadm", "-z", zoneName, "boot",
	     (char *) NULL);
      printf ("WE ARE HERE!!!\n");
    }
  wait (&exitStatus);

  return 0;
}

void
enterZone (const char *zoneName, int argc, char **argv)
{
  //we need to setup the arguments to be passed to zlogin
  //Basically forward devzone_enter's arguments to zlogin
  //So if incoming argv="devzone_enter" "a" "bbb" "cc cc"
  //outgoing zoneargv="zlogin" zoneName "a" "bbb" "cc cc" NULL
  //so zoneargv will have 2 more items than argv
  char **zoneargv;
  char zlogin_str[] = "zlogin";
  int i;

  setreuid (0, 0);

// Need to run zlogin -S on some more developmental setups
#ifdef USE_ZLOGIN_SAFEMODE
#define ZS_ARG_INC +1
#else
#define ZS_ARG_INC +0
#endif

  // enter the zone
  // TODO: add original program arguments to this exec function so
  //       that users can execute a function in the zone directly
  //       from the CLI
  zoneargv = malloc ((argc + 2 ZS_ARG_INC) * sizeof (char *));
  zoneargv[0] = malloc (strlen (zlogin_str) + 1);
  strcpy (zoneargv[0], zlogin_str);

#ifdef USE_ZLOGIN_SAFEMODE
  zoneargv[1] = "-S";
#endif

  zoneargv[1 ZS_ARG_INC] = malloc (strlen (zoneName) + 1);
  strcpy (zoneargv[1 ZS_ARG_INC], zoneName);

  //start from argv[1].. forget argv[0]=devzone_enter
  for (i = 1; i < argc; i++)
    {
      zoneargv[i + 1 ZS_ARG_INC] = argv[i];
    }

  //finally end with a NULL
  zoneargv[argc + 2] = NULL;
  execv ("/usr/sbin/zlogin", zoneargv);
}
